import { NgModule } from '@angular/core';
import { Route, RouterModule } from '@angular/router';
import { AuthTabEnum } from './modules/auth/enums/auth-tab.enum';
import { UserRoleEnum } from './modules/auth/enums/user-role.enum';
import { AuthGuard } from './modules/auth/services/auth/auth.guard';
import { NotFoundComponent } from './modules/shared/components/not-found/not-found.component';

export interface AppMenuEntry {
    label: string;
    icon: string;
    path?: string;
    matchExact?: boolean;
    level?: number;
    roles: UserRoleEnum[];
}

interface AppRoute extends Route {
    menuEntries?: AppMenuEntry[];
}

export const appRoutes: AppRoute[] = [
    {
        path: '',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.HOME',
                icon: 'fa fa-home',
                roles: [],
                matchExact: true,
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'auth',
        loadChildren: () => import('./modules/auth/auth.module').then(m => m.AuthModule),
        menuEntries: [
            {
                path: `auth/${AuthTabEnum.Login}`,
                label: 'AUTH.SIGN_IN',
                icon: 'fa fa-sign-in',
                roles: [ UserRoleEnum.None ],
                matchExact: true,
            },
            {
                path: `auth/${AuthTabEnum.Register}`,
                label: 'AUTH.SIGN_UP',
                icon: 'fa fa-sign-out',
                roles: [ UserRoleEnum.None ],
                matchExact: true,
            },
            {
                path: `auth/${AuthTabEnum.RestorePassword}`,
                label: 'AUTH.RESTORE_ACCOUNT',
                icon: 'fa fa-undo',
                roles: [ UserRoleEnum.None ],
                matchExact: true,
            },

        ],
    },
    {
        path: 'profile',
        loadChildren: () => import('./modules/profile/profile.module').then(m => m.ProfileModule),
        menuEntries: [
            {
                label: 'MENU.PROFILE',
                icon: 'fa fa-user',
                roles: [ UserRoleEnum.User ],
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'community',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.COMMUNITY',
                icon: 'fa fa-users',
                roles: [ UserRoleEnum.User ],
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'questions',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.QUESTIONS',
                icon: 'fa fa-question',
                roles: [ UserRoleEnum.User ],
                level: 1,
            }
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'messages',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.MESSAGES',
                icon: 'fa fa-comments',
                roles: [ UserRoleEnum.User ],
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'tests',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.TESTS',
                icon: 'fa fa-heartbeat',
                roles: [ UserRoleEnum.User ],
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'research',
        component: NotFoundComponent,
        menuEntries: [
            {
                label: 'MENU.RESEARCH',
                icon: 'fa fa-flask',
                roles: [ UserRoleEnum.User ],
            },
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
    {
        path: 'admin',
        loadChildren: () => import('./modules/admin/admin.module').then(m => m.AdminModule),
        menuEntries: [
            {
                label: 'MENU.ADMIN',
                icon: 'fa fa-id-card',
                roles: [ UserRoleEnum.Administrator, UserRoleEnum.Moderator ],
                matchExact: true,
            },
            {
                path: 'admin/users',
                label: 'MENU.USERS',
                icon: 'fa fa-users',
                roles: [ UserRoleEnum.Administrator, UserRoleEnum.Moderator ],
                level: 1,
            }
        ],
        canActivate: [ AuthGuard, ],
        canActivateChild: [ AuthGuard, ],
    },
];

@NgModule({
    imports: [RouterModule.forRoot(appRoutes)],
    exports: [RouterModule]
})
export class AppRoutingModule { }
