import { AfterContentInit, Component } from '@angular/core';
import { AppMenuEntry, appRoutes } from './app-routing.module';
import { AppService, WindowSize } from './modules/shared/services/app/app.service';
import { BrowserStorageService } from './modules/shared/services/browser-storage/browser-storage.service';
import { DomSanitizer, SafeStyle } from '@angular/platform-browser';
import { AuthService } from './modules/auth/services/auth/auth.service';
import { Router } from '@angular/router';
import { UserModel } from './modules/auth/models/user.model';
import { UserRoleEnum } from './modules/auth/enums/user-role.enum';

enum SidebarOpenEnum {
    Default,
    Open,
    Closed,
}

@Component({
    selector: 'app-root',
    templateUrl: './app.component.html',
    styleUrls: ['./app.component.scss']
})
export class AppComponent implements AfterContentInit {

    isMobile = true;
    title = 'CureNet';
    sidebarOpen = SidebarOpenEnum.Default;
    sidebarMode = 'over';
    defaultSidebarOpen = false;
    appRoutes = appRoutes;
    user: UserModel;
    loaded = false;
    dynamicToolbarComponents = [];
    roles: UserRoleEnum[] = [ UserRoleEnum.None ];

    get isSidebarOpen(): boolean {
        switch (this.sidebarOpen) {
            case SidebarOpenEnum.Open:
            return true;
            case SidebarOpenEnum.Closed:
            return false;
            default:
            return this.defaultSidebarOpen;
        }
    }

    constructor(
        private appService: AppService,
        private browserStorageService: BrowserStorageService,
        private sanitizer: DomSanitizer,
        private authService: AuthService,
        private router: Router,
    ) {
    }

    ngAfterContentInit(): void {
        this.configureResizeEvents();
        this.configureUserEvents();
        this.configureSidebarEvents();
    }

    canShowMenuEntry(menu: AppMenuEntry): boolean {
        return !menu.roles.some(i => !this.roles.includes(i));
    }

    configureUserEvents(): void {
        this.authService.checkUser().then(user => {
            this.setUser(user);
            this.loaded = true;
        }).catch(() => {
            this.setUser(null);
            this.loaded = true;
        });
        this.setUser(this.authService.getUser());
        this.authService.userChange.subscribe(user => {
            this.setUser(user);
        });
    }

    setUser(user: UserModel): void {
        if (!user && !this.user) {
            return;
        }
        if (user && this.user) {
            if (user.id === this.user.id && user.roles.join(',') === this.user.roles.join(',')) {
                return;
            }
        }
        this.user = user;
        if (this.user) {
            this.roles = this.user.roles;
        } else {
            this.roles = [ UserRoleEnum.None ];
        }
    }

    configureSidebarEvents(): void {
        const sidebarUserPrefference = this.browserStorageService.getItem('sidebar.open');
        if (sidebarUserPrefference !== null) {
            this.sidebarOpen = sidebarUserPrefference;
        }
        this.appService.dynamicToolbarComponentsChange.subscribe(components => {
            this.dynamicToolbarComponents = components;
        });
    }

    configureResizeEvents(): void {
        this.onResize(this.appService.size);
        this.appService.sizeChange.subscribe(size => {
            this.onResize(size);
        });
    }

    onResize(size: WindowSize): void {
        if (size === undefined) {
            return;
        }
        this.isMobile = size.isMobile;
        this.defaultSidebarOpen = !size.isMobile;
        this.sidebarMode = this.isMobile ? 'over' : 'side';
    }

    toggleSidebar(): void {
        this.sidebarOpen = this.isSidebarOpen ? SidebarOpenEnum.Closed : SidebarOpenEnum.Open;
        this.browserStorageService.setItem('sidebar.open', this.sidebarOpen);
    }

    onSidebarOpenedChange(opened: boolean): void {
        this.sidebarOpen = opened ? SidebarOpenEnum.Open : SidebarOpenEnum.Closed;
    }

    url(url: string): SafeStyle {
        return this.sanitizer.bypassSecurityTrustStyle(`url('${url}')`);
    }

    logout(): void {
        this.authService.logout();
        this.router.navigateByUrl('/');
    }
}
