import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { DomSanitizer, SafeStyle } from '@angular/platform-browser';
import { ActivatedRoute, ActivatedRouteSnapshot, ActivationStart, Data, Router, RoutesRecognized } from '@angular/router';
import { PersonModel } from 'src/app/modules/auth/models/person.model';
import { AuthService } from 'src/app/modules/auth/services/auth/auth.service';
import { ProfileTabEnum } from '../../enums/profile-tab.enum';
import { profileTabRoutes } from '../../profile-routing';
import { PersonService } from '../../service/person/person.service';

interface ProfileTab {
    tab: ProfileTabEnum;
    label: string;
    icon?: string;
}

@Component({
    selector: 'app-profile-edit',
    templateUrl: './profile-edit.component.html',
    styleUrls: ['./profile-edit.component.scss']
})
export class ProfileEditComponent implements AfterViewInit {

    profileTabRoutes = profileTabRoutes;
    currentTabName: string;
    person: PersonModel | null = null;
    selectedIndex = 0;
    defaultProfileTab = ProfileTabEnum.Basics;
    footerElement: ElementRef;
    loaded = false;
    @ViewChild('scrollHeader', {static: false}) scrollHeader: ElementRef;
    @ViewChild('scrollContent', {static: false}) scrollContent: ElementRef;
    headerHeight = 200;
    headerMove = 0;
    maxHeaderMove = 0;
    scrollWidth = 30;
    tabs: ProfileTab[] = [
        {
            tab: ProfileTabEnum.Basics,
            label: 'PROFILE.BASICS',
            icon: 'fa fa-id-badge',
        },
        {
            tab: ProfileTabEnum.Health,
            label: 'PROFILE.HEALTH',
            icon: 'fa fa-heartbeat',
        },
        {
            tab: ProfileTabEnum.Roles,
            label: 'PROFILE.ROLES',
            icon: 'fa fa-user',
        },
        {
            tab: ProfileTabEnum.Credentials,
            label: 'PROFILE.PASSWORD',
            icon: 'fa fa-unlock-alt',
        },
        {
            tab: ProfileTabEnum.Settings,
            label: 'PROFILE.SITE_SETTINGS',
            icon: 'fa fa-sliders',
        },
    ];

    constructor(
        private sanitizer: DomSanitizer,
        authService: AuthService,
        private activatedRoute: ActivatedRoute,
        private router: Router,
        private personService: PersonService,
    ) {
        const personId = parseInt(activatedRoute.snapshot.params.personId, 10);
        if (personId) {
            this.loadPerson(personId);
        } else {
            this.loadPerson(authService.getUser().person.id);
        }

        this.onRouteChange(activatedRoute.snapshot.firstChild?.data);
        this.router.events.subscribe(event => {
            if (event instanceof RoutesRecognized) {
                console.log({event});
            }
        });
    }

    loadPerson(personId: number) {
        this.person = undefined;
        this.loaded = false;
        this.personService.get(personId).subscribe(person => {
            this.person = person;
            this.loaded = true;
        });
    }

    updateScroller() {
        this.scrollHeader.nativeElement.style.height = '0px';
        this.headerHeight = this.scrollHeader.nativeElement.scrollHeight;
        this.maxHeaderMove = this.scrollHeader.nativeElement.firstChild.scrollHeight;
        this.scrollWidth = this.scrollContent.nativeElement.parentElement.scrollWidth - this.scrollContent.nativeElement.scrollWidth;
    }

    ngAfterViewInit(): void {
        this.loaded = true;
        setTimeout(() => {
            this.updateScroller();
        }, 50);
    }

    activated(component) {
        if (component.footerElement) {
            this.footerElement = component.footerElement;
        }
        setTimeout(() => {
            if (component.footerElement) {
                this.footerElement = component.footerElement;
            }
            this.updateScroller();
        }, 50);
    }

    deactivated(component) {
        this.footerElement = undefined;
        setTimeout(() => {
            this.updateScroller();
        }, 50);
    }

    onscrol(e) {
        this.headerMove = Math.min(e.target.scrollTop, this.maxHeaderMove);
    }

    onRouteChange(data: Data): void {
        this.currentTabName = undefined;
        if (data?.tab) {
            const tab = data.tab as ProfileTabEnum;
            const tabIndex = this.tabs.findIndex(i => i.tab === tab);
            if (tabIndex !== -1) {
                this.selectedIndex = tabIndex;
                this.currentTabName = this.tabs[tabIndex].label;
            }
        } else {
            const index = this.tabs.findIndex(i => i.tab === this.defaultProfileTab);
            if (index !== -1) {
                this.selectedIndexChange(index);
            }
        }
    }

    selectedIndexChange(index: number): void {
        this.selectedIndex = index;
        this.currentTabName = undefined;
        const tab = this.tabs[index];
        if (tab) {
            const route = this.profileTabRoutes.find(i => i.data.tab === tab.tab);
            this.currentTabName = tab.label;
            if (route) {
                this.router.navigate([route.path], {
                    relativeTo: this.activatedRoute,
                });
            }
        }
    }

    url(url: string): SafeStyle {
        return this.sanitizer.bypassSecurityTrustStyle(`url('${url}')`);
    }
}
